from setuptools import setup, find_packages

setup(
    name="pixelyai_core",
    version="0.0.1",
    author="Erfan Zare Chavoshi",
    author_email="erfanzare82@eyahoo.com",
    description="Serve Utilises of PixelyAI",
    url="https://github.com/erfanzar/PixelyAI",
    packages=find_packages("src/python"),
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    keywords="machine learning, deep learning, pytorch, jax, flax, ggml, c++",
    install_requires=[
        "typing~=3.7.4.3",
        "ipython~=8.17.2",
        "tqdm~=4.64.1",
        "pydantic==2.5.3",
        "setuptools~=68.1.2",
        "gradio~=4.12.0",
        "uvicorn~=0.23.2",
        "fastapi~=0.104.1",
        "pydantic-core==2.14.6",
    ],
    python_requires=">=3.8",
    package_dir={"": "src/python"},
    password="PixelyAICore"
)
